// package: l2
// file: avvyprotocol/l2_api.proto

var avvyprotocol_l2_api_pb = require("../avvyprotocol/l2_api_pb");
var grpc = require("@improbable-eng/grpc-web").grpc;

var L2Service = (function () {
  function L2Service() {}
  L2Service.serviceName = "l2.L2Service";
  return L2Service;
}());

L2Service.Request = {
  methodName: "Request",
  service: L2Service,
  requestStream: false,
  responseStream: false,
  requestType: avvyprotocol_l2_api_pb.L2Request,
  responseType: avvyprotocol_l2_api_pb.L2Response
};

L2Service.Stream = {
  methodName: "Stream",
  service: L2Service,
  requestStream: true,
  responseStream: false,
  requestType: avvyprotocol_l2_api_pb.L2Request,
  responseType: avvyprotocol_l2_api_pb.L2Response
};

L2Service.EventBus = {
  methodName: "EventBus",
  service: L2Service,
  requestStream: false,
  responseStream: true,
  requestType: avvyprotocol_l2_api_pb.Empty,
  responseType: avvyprotocol_l2_api_pb.L2Event
};

L2Service.ContentBus = {
  methodName: "ContentBus",
  service: L2Service,
  requestStream: false,
  responseStream: true,
  requestType: avvyprotocol_l2_api_pb.Empty,
  responseType: avvyprotocol_l2_api_pb.L2Content
};

exports.L2Service = L2Service;

function L2ServiceClient(serviceHost, options) {
  this.serviceHost = serviceHost;
  this.options = options || {};
}

L2ServiceClient.prototype.request = function request(requestMessage, metadata, callback) {
  if (arguments.length === 2) {
    callback = arguments[1];
  }
  var client = grpc.unary(L2Service.Request, {
    request: requestMessage,
    host: this.serviceHost,
    metadata: metadata,
    transport: this.options.transport,
    debug: this.options.debug,
    onEnd: function (response) {
      if (callback) {
        if (response.status !== grpc.Code.OK) {
          var err = new Error(response.statusMessage);
          err.code = response.status;
          err.metadata = response.trailers;
          callback(err, null);
        } else {
          callback(null, response.message);
        }
      }
    }
  });
  return {
    cancel: function () {
      callback = null;
      client.close();
    }
  };
};

L2ServiceClient.prototype.stream = function stream(metadata) {
  var listeners = {
    end: [],
    status: []
  };
  var client = grpc.client(L2Service.Stream, {
    host: this.serviceHost,
    metadata: metadata,
    transport: this.options.transport
  });
  client.onEnd(function (status, statusMessage, trailers) {
    listeners.status.forEach(function (handler) {
      handler({ code: status, details: statusMessage, metadata: trailers });
    });
    listeners.end.forEach(function (handler) {
      handler({ code: status, details: statusMessage, metadata: trailers });
    });
    listeners = null;
  });
  return {
    on: function (type, handler) {
      listeners[type].push(handler);
      return this;
    },
    write: function (requestMessage) {
      if (!client.started) {
        client.start(metadata);
      }
      client.send(requestMessage);
      return this;
    },
    end: function () {
      client.finishSend();
    },
    cancel: function () {
      listeners = null;
      client.close();
    }
  };
};

L2ServiceClient.prototype.eventBus = function eventBus(requestMessage, metadata) {
  var listeners = {
    data: [],
    end: [],
    status: []
  };
  var client = grpc.invoke(L2Service.EventBus, {
    request: requestMessage,
    host: this.serviceHost,
    metadata: metadata,
    transport: this.options.transport,
    debug: this.options.debug,
    onMessage: function (responseMessage) {
      listeners.data.forEach(function (handler) {
        handler(responseMessage);
      });
    },
    onEnd: function (status, statusMessage, trailers) {
      listeners.status.forEach(function (handler) {
        handler({ code: status, details: statusMessage, metadata: trailers });
      });
      listeners.end.forEach(function (handler) {
        handler({ code: status, details: statusMessage, metadata: trailers });
      });
      listeners = null;
    }
  });
  return {
    on: function (type, handler) {
      listeners[type].push(handler);
      return this;
    },
    cancel: function () {
      listeners = null;
      client.close();
    }
  };
};

L2ServiceClient.prototype.contentBus = function contentBus(requestMessage, metadata) {
  var listeners = {
    data: [],
    end: [],
    status: []
  };
  var client = grpc.invoke(L2Service.ContentBus, {
    request: requestMessage,
    host: this.serviceHost,
    metadata: metadata,
    transport: this.options.transport,
    debug: this.options.debug,
    onMessage: function (responseMessage) {
      listeners.data.forEach(function (handler) {
        handler(responseMessage);
      });
    },
    onEnd: function (status, statusMessage, trailers) {
      listeners.status.forEach(function (handler) {
        handler({ code: status, details: statusMessage, metadata: trailers });
      });
      listeners.end.forEach(function (handler) {
        handler({ code: status, details: statusMessage, metadata: trailers });
      });
      listeners = null;
    }
  });
  return {
    on: function (type, handler) {
      listeners[type].push(handler);
      return this;
    },
    cancel: function () {
      listeners = null;
      client.close();
    }
  };
};

exports.L2ServiceClient = L2ServiceClient;

